{Application 'BHIMPUTE' logic file generated by CSPro }
PROC GLOBAL 
  file BHimpute;

  { Event table }
  array type     (30);
  array norder   (30);
  array twincode (30);
  array surv     (30);
  array sflag    (30);
  array smonth   (30);
  array errflag  (30);
  array lcmc     (30);
  array ucmc     (30);
  array cmc      (30);
  array interv   (30);
  array cdelay   (30);
  array duramen  (30);
  array durabst  (30);
  array durbf    (30);
  array durmeth  (30);
  array flagamen (30);
  array flagabst (30);
  array calrow   (30);

  array flagbf   (30);
  array flagmeth (30);
  array flagdth  (30);
  array deathl   (30);
  array deathu   (30);
  array lstdu    (30);
  array lstdn    (30);

  numeric ismonth, isyear, isage, flag, nEvents, LcmcB, LcmcM, UcmcB, UcmcM;
  numeric mind, minai, mindb, minam, minamm, minau, minaum, minas, minab, mincd, minpd, lmax, lmin;
  numeric maxd, maxai, maxdb;
  numeric maxelig, maxevent, inunion, evermarr;
  numeric doimp, emsample, ignorey;
  numeric temp, i, j, di, run1, t, x, chadj, k, e;
  numeric agedu, agedn, aged, umin, cebnew, ntwin, xdeath;
  numeric x1, x2, eno, jinterv, gap, gap2, xgap, du, dn, dmonths, xdatemin, xdatemax;
  numeric calive, cdead, bsurv, bdead, bunknown;
  alpha( 1 ) asterisk, spaces;
  alpha( 5 ) sdeath;
  alpha(120) linestr;

  { Revised function to adjust dates based on age }
  function ndjlba(lowera,uppera,lowerb,upperb,agetofit)
    temp = lowerb - 12 * (agetofit + 1);
    if temp > uppera then
      temp = -1
    else
      if temp < uppera then
        temp = temp + 1
      endif;
      if lowera > temp then
        temp = lowera
      endif
    endif;
    ndjlba = temp
  end;

  function NAtoZero(xvar);
    if xvar = notappl then
      xvar = 0
    endif;
    NAtoZero = xvar;
  end;
  
  function misstozero(xvar);
    if xvar = notappl | xvar = missing | xvar = 99 then
      xvar = 0
    endif;
    misstozero = xvar;
  end;

  function valid(xvar);
    valid = (!special(xvar) & xvar <= 96)
  end;

  function validyr(xvar);
    validyr = (!special(xvar) & xvar <= 9996)
  end;

  function cmc2m(xcmc);
    cmc2m = (xcmc-1)%12+1;
  end;

  function cmc2y(xcmc);
    cmc2y = int((xcmc-1)/12)+1900;
  end;

PROC BHEXTRACT_FF
preproc
  set behavior() export (SPSS SubitemOnly);

  { Initializing the application before the first questionnaire }
  seed(101);          { Seed to start the random number generator }

  { Set the applications minimum and maximum parameters }
  minai    = 15;        { !!! Minimum age at interview                }
  maxai    = 49;        { !!! Maximum age at interview                }
  minam    = 48;       { !!! Minimum age at marriage                 }
  minab    = 96;       { !!! Minimum age at birth                    }
  minpd    = 7;         { !!! Minimum pregnancy duration              }
  mincd    = 2;         { !!! Minimum conception delay after birth    }
  maxevent = 30;        { !!! Maximum number of events                }
  doimp    = 1;         { 1 if doing imputation - 0 if no imputation  }
  ignorey  = 0;         { 1 if ignoring year when age + year = interview year
                          0 if using year when age + year = interview year }
  emsample = 0;         { !!! Ever married sample - 0=No, 1=Yes       }
  run1 = 1;

  do i = 1 while i <= maxevent
    lstdu(i) = 2;               { Units  for hot deck date of death }
    lstdn(i) = int((i-1)/10);   { Number for hot deck date of death }
  enddo;

PROC BHEXTRACT_QUEST
preproc

  di = cmcode( WM6M(1), WM6Y(1) );
  
if WM7(1) = 1 then { Complete interview }
  
  mindb = di - (maxai+1)*12 + 1;      { Minimum date of birth - women }
  maxdb = di - minai*12;              { Maximum date of birth - women }

  { Initialize event table }
  do i = 1 while i <= maxevent
    norder(i)  = default;     { Birth order of children }
    twincode(i)= default;     { Twin code for children }
    surv(i)    = default;     { Survival status for children }
    deathl(i)  = default;     { Age at death of child - lower limit }
    deathu(i)  = default;     { Age at death of child - upper limit }
    sflag(i)   = default;     { Status code flag }
    errflag(i) = default;     { Error flag }
    flagdth(i) = 0;           { Flag for age at death of the child }
  enddo;
  inunion = (MA1(1) = 1);

  { Date of birth of the woman }

  { Check either year of birth or age given for all women }
  if !validyr(WM8Y(1)) & !valid(WM9(1)) then
    if run1 then errmsg(1060,WM8M(1),WM8Y(1),WM9(1)); endif;
  endif;

  { Set up event table entry for date of birth of woman }
  nevents = 1;
  ismonth = valid(WM8M(1));
  isyear  = validyr(WM8Y(1));
  isage   = valid(WM9(1));
  if ismonth then
    smonth(1) = WM8M(1);
  else
    smonth(1) = 0;
  endif;
  box isyear : ismonth : isage => flag;
           1 :       1 :       => 1;
             :       1 :     1 => 2;
           1 :         :     1 => 3;
           1 :         :       => 5;
             :         :     1 => 6;
             :       1 :       => 7;
             :         :       => 8;
  endbox;
  if ignorey & flag = 3 & WM8Y(1) + WM9(1) = WM6Y(1) then
    flag = 4
  endif;
  sflag(1)   = flag;
  errflag(1) = notappl;

  { Initial ranges for CMC date of birth }
  if validyr(WM8Y(1)) & flag <> 4 then
    lcmcb = setlb(WM8M(1),WM8Y(1),0);
    ucmcb = setub(WM8M(1),WM8Y(1),9999);
    if lcmcb < mindb then
      if ucmcb < mindb then
        errmsg(1063,WM8M(1),WM8Y(1),WM6M(1),WM6Y(1),cmc2m(mindb),cmc2y(mindb),cmc2m(maxdb),cmc2y(maxdb));
        errflag(1) = default;
        ucmcb = mindb;
      endif;
      lcmcb = mindb;
    endif;
    if ucmcb > maxdb then
      if lcmcb > maxdb then
        errmsg(1063,WM8M(1),WM8Y(1),WM6M(1),WM6Y(1),cmc2m(mindb),cmc2y(mindb),cmc2m(maxdb),cmc2y(maxdb));
        errflag(1) = default;
        lcmcb = maxdb;
      endif;
      ucmcb = maxdb;
    endif
  else
    lcmcb = mindb;
    ucmcb = maxdb;
  endif;

  type(1) = 1;
  interv(1) = 0;
  cdelay(1) = 0;

{ -------------------------------------------------------------------------- }

  { Adjust ranges for CMC date of birth based on age }
  if (WM9(1) in minai:maxai) then
    t = ndjlba(lcmcb,ucmcb,di,di,WM9(1));
    if t < 0 then
      errmsg(1062,WM8M(1),WM8Y(1),lcmcb,ucmcb,WM9(1),WM6M(1),WM6Y(1),di);
      errflag(1) = default;
      { Need to give priority to age instead of date of birth when inconsistent }
      t = adjlba(mindb,maxdb,di,di,WM9(1));
      if t < 0 then
        errmsg("Date of birth and age still a problem 1: WM8=%02d/%04d, WM9=%02d",WM8M(1),WM8Y(1),WM9(1));
      else
        lcmcb = t;
        ucmcb = maxdb;
        t = adjuba(lcmcb,ucmcb,di,di,WM9(1));
        if t < 0 then
          errmsg("Date of birth and age still a problem 2: WM8=%02d/%04d, WM9=%02d",WM8M(1),WM8Y(1),WM9(1));
        else
          ucmcb = t;
        endif;
      endif;
    else
      lcmcb = t;
    endif;
    t = adjuba(lcmcb,ucmcb,di,di,WM9(1));
    if t < 0 then
      errmsg(1062,WM8M(1),WM8Y(1),lcmcb,ucmcb,WM9(1),WM6M(1),WM6Y(1),di);
      errflag(1) = default;
      { Need to give priority to age instead of date of birth when inconsistent }
      t = adjlba(mindb,maxdb,di,di,WM9(1));
      if t < 0 then
        errmsg("Date of birth and age still a problem 3: WM8=%02d/%04d, WM9=%02d",WM8M(1),WM8Y(1),WM9(1));
      else
        lcmcb = t;
        ucmcb = maxdb;
        t = adjuba(lcmcb,ucmcb,di,di,WM9(1));
        if t < 0 then
          errmsg("Date of birth and age still a problem 4: WM8=%02d/%04d, WM9=%02d",WM8M(1),WM8Y(1),WM9(1));
        else
          ucmcb = t;
        endif;
      endif;
    else
      ucmcb = t;
    endif;
    x = di - (WM9(1)*12 + 11);
    if lcmcb < x & x <= ucmcb then
      lcmcb = x
    endif
  endif;

{ -------------------------------------------------------------------------- }

  { For month only without year, make adjustment }
  if ismonth & !isyear then
    x = int((lcmcb - 1)/12) * 12 + smonth(1);
    if x < lcmcb then
      x = x + 12
    endif;
    if x > ucmcb then
      if run1 then errmsg(1064,lcmcb,ucmcb,cmc2m(lcmcb),cmc2y(lcmcb),cmc2m(ucmcb),cmc2y(ucmcb),smonth(1)) endif;
    else
      lcmcb = x
    endif;
    x = int((ucmcb - 1)/12) * 12 + smonth(1);
    if x > ucmcb then
      x = x - 12
    endif;
    if x < lcmcb then
      if run1 then errmsg(1064,lcmcb,ucmcb,cmc2m(lcmcb),cmc2y(lcmcb),cmc2m(ucmcb),cmc2y(ucmcb),smonth(1)) endif;
    else
      ucmcb = x
    endif;
  endif;

{ -------------------------------------------------------------------------- }

  { Date of marriage }
  evermarr = (MA1(1) = 1 | MA3(1) = 1);
  chadj = 1;                    { Adjustment for children in event table }
  if evermarr then              { added to line number of child          }
    chadj = 2;
    nevents = nevents + 1;
    ismonth = valid(MA6M(1));
    isyear  = validyr(MA6Y(1));
    isage   = valid(MA8(1));
    if ismonth then
      smonth(2) = MA6M(1);
    else
      smonth(2) = 0;
    endif;
    box isyear : ismonth : isage => flag;
             1 :       1 :       => 1;
               :       1 :     1 => 2;
             1 :         :     1 => 3;
             1 :         :       => 5;
               :         :     1 => 6;
               :       1 :       => 7;
               :         :       => 8;
    endbox;
    sflag(2)   = flag;
    errflag(2) = notappl;
    type(2)    = 2;
    { No year of marriage given, check if age given }
    if flag in 7,8 then
      if run1 then errmsg(5135,MA8(1),MA6M(1),MA6Y(1)) endif;
    endif;

    { Initial ranges for date  of marriage }
    if validyr(MA6Y(1)) then
      lcmcm = setlb(MA6M(1),MA6Y(1),0);
      ucmcm = setub(MA6M(1),MA6Y(1),9999);
      if ucmcm > di then ucmcm = di endif;
    else
      lcmcm = lcmcb + minam;
      ucmcm = di;
    endif;

{ -------------------------------------------------------------------------- }

    { Adjust ranges for date of marriage based on age at marriage }
    if valid(MA8(1)) then

      { Adjust functions used with dates upside down to allow
        adjustment of date of marriage based on age at marriage
        rather than adjusting date of birth }
      t = adjuba(di-ucmcm,di-lcmcm,di-ucmcb,di-lcmcb,MA8(1));
      if t < 0 then
        errmsg(5132,WM6M(1),WM6Y(1),WM9(1),WM8M(1),WM8Y(1),MA8(1),MA6M(1),MA6Y(1));
        errflag(2) = default;
      else
        lcmcm = di-t;
      endif;
      t = ndjlba(di-ucmcm,di-lcmcm,di-ucmcb,di-lcmcb,MA8(1));
      if t < 0 then
        errmsg(5132,WM6M(1),WM6Y(1),WM9(1),WM8M(1),WM8Y(1),MA8(1),MA6M(1),MA6Y(1));
        errflag(2) = default;
      else
        ucmcm = di-t;
      endif;

{ -------------------------------------------------------------------------- }

    { Adjust ranges for date of birth based on age at marriage }
      if errflag(2) <> default then
        t = ndjlba(lcmcb,ucmcb,lcmcm,ucmcm,MA8(1));
        if t < 0 then
          errmsg(5131,WM6M(1),WM6Y(1),WM9(1),WM8M(1),WM8Y(1),MA8(1),MA6M(1),MA6Y(1));
          errflag(2) = default;
        else
          lcmcb = t;
        endif;
        t = adjuba(lcmcb,ucmcb,lcmcm,ucmcm,MA8(1));
        if t < 0 then
          errmsg(5131,WM6M(1),WM6Y(1),WM9(1),WM8M(1),WM8Y(1),MA8(1),MA6M(1),MA6Y(1));
          errflag(2) = default;
        else
          ucmcb = t;
        endif
      endif

    endif;

    { Check date of marriage not after date of interview }
    if lcmcm > di then
      errmsg(5130,WM6M(1),WM6Y(1),WM8M(1),WM8Y(1),WM9(1),MA6M(1),MA6Y(1),MA8(1));
      errflag(2) = default;
    elseif ucmcm > di then
      ucmcm = di
    endif;

{ -------------------------------------------------------------------------- }

    { For month only without year, make adjustment }
    if ismonth & !isyear then
      x = int((lcmcm - 1)/12) * 12 + smonth(2);
      if x < lcmcm then
        x = x + 12
      endif;
      if x > ucmcm then
        errmsg(5126,lcmcm,ucmcm,cmc2m(lcmcm),cmc2y(lcmcm),cmc2m(ucmcm),cmc2y(ucmcm),smonth(2));
      else
        lcmcm = x
      endif;
      x = int((ucmcm - 1)/12) * 12 + smonth(2);
      if x > ucmcm then
        x = x - 12
      endif;
      if x < lcmcm then
        errmsg(5126,lcmcm,ucmcm,cmc2m(lcmcm),cmc2y(lcmcm),cmc2m(ucmcm),cmc2y(ucmcm),smonth(2));
      else
        ucmcm = x
      endif;
    endif;

{ -------------------------------------------------------------------------- }

    { Adjust for minimum age at marriage }
    t = adjlbi(lcmcb,ucmcb,lcmcm,ucmcm,minam);
    if t < 0 then
      if run1 then errmsg(5121,int(minam/12),WM8M(1),WM8Y(1),WM9(1),MA6M(1),MA6Y(1),MA8(1)) endif;
      errflag(2) = default;
    else
      lcmcm = t;
    endif;

    { Check date of marriage not after date of interview }
    if lcmcm > di then
      errmsg(5120,MA6M(1),MA6Y(1),MA8(1),WM6M(1),WM6Y(1),WM8M(1),WM8Y(1),WM9(1));
      errflag(2) = default;
    elseif ucmcm > di then
      ucmcm = di
    endif;

    lcmc(2) = lcmcm;
    ucmc(2) = ucmcm;
    interv(2) = minam;
    cdelay(2) = 0;

  endif;

  lcmc(1) = lcmcb;
  ucmc(1) = ucmcb;

{ -------------------------------------------------------------------------- }

  cebnew = totocc(MODBH_EDT);

  { Birth history }
  calive = misstozero(CM4A(1))+misstozero(CM4B(1))+misstozero(CM6A(1))+misstozero(CM6B(1));
  cdead = misstozero(CM8A(1))+misstozero(CM8B(1));
  if calive+cdead <> CM9(1) then errmsg("Count of births inconsistent: CM4=%02d %02d CM6=%02d %02d CM8=%02d %02d CM9=%02d",CM4A(1),CM4B(1),CM6A(1),CM6B(1),CM8A(1),CM8B(1),CM9(1)) endif;
  bsurv = 0;
  bdead = 0;
  bunknown = 0;
  for i in modbh_edt do
    blankrec(i) = (special(BH5));
    if BH5 = 1 then bsurv = bsurv + 1
    elseif BH5 = 2 then bdead = bdead + 1
    else bunknown = bunknown + 1
    endif;
  enddo;
  for i in modbh_edt do
    impBH5(i) = BH5;
    if not BH5 in 1:2 then
      if bsurv < calive & bdead < cdead then
        impBH5(i) = random(1,2);
        if impBH5(i) = 1 then bsurv = bsurv + 1
        elseif impBH5(i) = 2 then bdead = bdead + 1
        endif;
      elseif bsurv < calive then
        impBH5(i) = 1;
        bsurv = bsurv + 1
      elseif bdead < cdead then
        impBH5(i) = 2;
        bdead = bdead + 1
      else
        impBH5(i) = random(1,2);
        if impBH5(i) = 1 then bsurv = bsurv + 1
        elseif impBH5(i) = 2 then bdead = bdead + 1
        endif;
      endif;
    endif;
    BH5 = impBH5(i);
  enddo;

  { Birth history }
  for i in modbh_edt do
  
    { set values as valid values where possible }
    if special(BH1) then BH1 = 99 endif;
    if not BH2 in 1,2 then BH2 = 9 endif;
    if special(BH3) | BH3 = 8 then BH3 = 9  endif;
    if not BH4M in 1:12 then BH4M = 99 endif;
    if not BH4Y in 1966:2006 then BH4Y = 9999 endif;
    if not BH5 in 1:2 then BH5 = 9 endif;
    if BH6 in 0:40 and BH5 = 9 then BH5 = 1 endif;
    if BH5 = 1 and not BH6 in 0:40 then BH6 = 99 endif;
  
    nevents    = nevents + 1;
    j          = nevents;
    type(j)    = 3;
    norder(j)  = i;
    twincode(j)= 0;
    surv(j)    = (BH5 = 2) + 1;
    deathl(j)  = 0;
    deathu(j)  = 0;
    durabst(j) = 0;
    duramen(j) = 0;
    durbf(j)   = 0;
    durmeth(j) = 0;
    flagabst(j)= 0;
    flagamen(j)= 0;
    flagbf(j)  = 0;
    flagmeth(j)= 0;
    smonth(j)  = 0;
    calrow(j)  = 0;

    isyear = 0; ismonth = 0; isage = 0;
    if valid(BH4M) then
      smonth(j) = BH4M;
      ismonth = 1
    endif;
    { Initialize logical ranges for date of birth of child }
    if validyr(BH4Y) then
      isyear = 1;
      lcmc(j) = setlb(BH4M,BH4Y,0);
      ucmc(j) = setub(BH4M,BH4Y,9999);
    else
      lcmc(j) = mindb + minab;
      ucmc(j) = di;
    endif;
    if valid(BH6) then
      isage = 1;
    endif;

    box isyear : ismonth : isage => flag;
             1 :       1 :       => 1;
               :       1 :     1 => 2;
             1 :         :     1 => 3;
             1 :         :       => 5;
               :         :     1 => 6;
               :       1 :       => 7;
               :         :       => 8;
    endbox;
    if ignorey & flag = 3 & BH4Y + BH6 = WM6Y(1) then
      flag = 4;
      lcmc(j) = mindb + minab;
      ucmc(j) = di;
    endif;
    sflag(j)   = flag;
    errflag(j) = notappl;

{ -------------------------------------------------------------------------- }

    { Set up twin code variable coded as follows:
      0 - Single birth
      1 - 1st of multiple
      2 - 2nd of multiple
      3 - 3rd of multiple
      etc.
      This variable easier to use in controlling twins.
    }

    if BH2 = 1 or BH2 = 9 then         { Single birth }
      twincode(j) = 0
    elseif BH1 = 1 then     { First birth }
      twincode(j) = 1
    else
      twincode(j) = twincode(j-1) + 1
    endif;


    { Update twin code variable in case of two pairs of twins together }
    { Twincode is 0-Single birth, 1-1st of multiple, 2-2nd of multiple, ... }
    if BH2 = 2 & i > 2 & twincode(j-1) >= 2 then
      if (BH4Y <> BH4Y(i-1) | BH4M <> BH4M(i-1)) then
        twincode(j) = 1
      endif
    endif;

{ -------------------------------------------------------------------------- }

    { Adjust ranges for date of birth of child based on age of child }
    if BH5 <> 2 then
      { Living children }
      if valid(BH6) then
        { Lower bound of CMC date of birth }
        t = ndjlba(lcmc(j),ucmc(j),di,di,BH6);
        if t < 0 then
          errmsg(2171,i,BH4M,BH4Y,WM6M(1),WM6Y(1),BH6);
          errflag(j) = default;
        else
          lcmc(j) = t;
        endif;
        { Upper bound of CMC date of birth }
        t = adjuba(lcmc(j),ucmc(j),di,di,BH6);
        if t < 0 then
          errmsg(2171,i,BH4M,BH4Y,WM6M(1),WM6Y(1),BH6);

          errflag(j) = default;
        else
          ucmc(j) = t;
        endif;
        x = di - (BH6*12 + 11);
        if lcmc(j) < x & x <= ucmc(j) then
          lcmc(j) = x
        endif
      endif;

    else

      { Re calculate age at death variables }

      { Children who have died }
      agedu = BH9A;         { age at death - units }
      agedn = BH9B;         { age at death - number }

      { Check maximum and mininum values }
      box agedu => maxd;
              1 => 30;       { 0-30 days   }
              2 => 23;       { 1-23 months }
              3 => 40;       { 2-40 years  }
                => 99;
      endbox;
      mind = agedu-1;
      if agedu = 9 then agedu = 0 endif;
      if !(agedn in mind:maxd) then
        if run1 then errmsg(2191,i,agedn,mind,maxd,agedu) summary endif;
        flagdth(i+chadj) = 6;
      endif;

      { Check age at death plausible considering date of birth }
      box agedu: agedn   => aged;
               : missing => 999;
               : >= 97   => 999;
             1 :         => int(agedn/30);
             2 :         => agedn;
             3 :         => agedn*12;
               :         => 999;
      endbox;
      if aged <> 999 then
        if lcmc(j) + aged > di then
          if run1 then errmsg(2192,i,BH4M,BH4Y,BH9A,BH9B,WM6M(1),WM6Y(1)) endif;
          errflag(j) = default;
          flagdth(j) = 1;
        elseif ucmc(j) + aged > di then
          ucmc(j) = di - aged
        endif;
        { Now make age at death as late as possible for those with
          age at death reported in years }
        deathl(j) = aged;               { lower limit of age at death }
        deathu(j) = aged;               { upper limit of age at death }
        if agedu = 3 then               { Age at death reported in years }
          deathu(j) = aged + 11;
          if flagdth(j) = 0 & lcmc(j) + deathu(j) > di then
            deathu(j) = di - lcmc(j)
          endif;
        endif
      else
        deathl(j) = 0;
        deathu(j) = di - lcmc(j);
      endif
    endif;

{ -------------------------------------------------------------------------- }

    { For month only without year, make adjustment }
    if ismonth & !isyear then
      x = int((lcmc(j) - 1)/12) * 12 + smonth(j);
      if x < lcmc(j) then
        x = x + 12
      endif;
      if x > ucmc(j) then
        if run1 then errmsg(2172,i,lcmc(j),ucmc(j),cmc2m(lcmc(j)),cmc2y(lcmc(j)),cmc2m(ucmc(j)),cmc2y(ucmc(j)),smonth(j)) endif;
      else
        lcmc(j) = x
      endif;
      x = int((ucmc(j) - 1)/12) * 12 + smonth(j);
      if x > ucmc(j) then
        x = x - 12
      endif;
      if x < lcmc(j) then
        if run1 then errmsg(2172,i,lcmc(j),ucmc(j),cmc2m(lcmc(j)),cmc2y(lcmc(j)),cmc2m(ucmc(j)),cmc2y(ucmc(j)),smonth(j)) endif;

      else
        ucmc(j) = x
      endif;
    endif;

{ -------------------------------------------------------------------------- }

    { Set minimum intervals }
    cdelay(j) = 0;
    if twincode(j) > 1 then
      interv(j) = 0;
    else
      if i = 1 then             { first birth }
        if j = 3 then           { woman ever married }
          interv(j) = 0
        else                    { woman never married }
          interv(j) = minab
        endif
      else                      { subsequent births }
        interv(j) = minpd;      { minimum pregnancy duration }
        cdelay(j) = mincd;      { minimum conception delay }
      endif
    endif;

  enddo;




{ -------------------------------------------------------------------------- }


  { Equalizing twins }
  umin = 9999;
  lmax = 0;
  ntwin = 0;
  do i = chadj+1 while i <= cebnew+chadj
    j = i + 1;
    if special(twincode(j)) | twincode(j) <= 1 then
      if umin < lmax then
        errmsg(9920,j-1,lmax,umin,cmc2m(lmax),cmc2y(lmax),cmc2m(umin),cmc2y(umin))
      else
        do k = 1 while k <= ntwin
          lcmc(j-k) = lmax;
          ucmc(j-k) = umin;
        enddo;
      endif;
      umin = 9999;
      lmax = 0;
      ntwin = 0;
    endif;
    if !special(twincode(j)) & twincode(j) >= 1 then
      ntwin = ntwin + 1;
      if lcmc(j) > lmax then lmax = lcmc(j) endif;
      if ucmc(j) < umin then umin = ucmc(j) endif;
    endif;
  enddo;

{ -------------------------------------------------------------------------- }

  { Date of interview event and open birth interval }
  nevents = nevents + 1;
  lcmc(nevents)   = di;
  ucmc(nevents)   = di;
  type(nevents)   = 6;
  sflag(nevents)  = 1;
  smonth(nevents) = 0;
  errflag(nevents)= notappl;
  interv(nevents) = 0;
  cdelay(nevents) = 0;

{ -------------------------------------------------------------------------- }

  if cebnew then
    { forward checking of date of first birth }
    i = chadj + 1;
    t = adjlbi(lcmc(1),ucmc(1),lcmc(i),ucmc(i),minab);
    if t < 0 then
      errmsg(9902,1,lcmc(1),ucmc(1),cmc2m(lcmc(1)),cmc2y(lcmc(1)),cmc2m(ucmc(1)),cmc2y(ucmc(1)),
                    i,lcmc(i),ucmc(i),cmc2m(lcmc(i)),cmc2y(lcmc(i)),cmc2m(ucmc(i)),cmc2y(ucmc(i)),minab);
      errflag(i) = default;
    else
      lcmc(i) = t;
      { backward checking of date of first birth }
      t = adjubi(lcmc(1),ucmc(1),lcmc(i),ucmc(i),minab);
      if t < 0 then
        errmsg(9902,i,lcmc(i),ucmc(i),cmc2m(lcmc(i)),cmc2y(lcmc(i)),cmc2m(ucmc(i)),cmc2y(ucmc(i)),
                      1,lcmc(1),ucmc(1),cmc2m(lcmc(1)),cmc2y(lcmc(1)),cmc2m(ucmc(1)),cmc2y(ucmc(1)),minab);
        errflag(1) = default;
      else
        ucmc(1) = t;
      endif;
    endif;
  endif;

{ -------------------------------------------------------------------------- }

  { forward checking of minimum interval }
  do i = 1 while i <= nevents - 1
    j = i + 1;
    t = adjlbi(lcmc(i),ucmc(i),lcmc(j),ucmc(j),interv(j)+cdelay(j));
    if t < 0 then
        x1 = type(i);
        x2 = type(j);
        box x1 : x2 => eno;
             1 :  2 => 9901;    { Between birth and first union }
             1 :  3 => 9902;    { Between birth and first child }
             2 :  3 => 9903;    { Between union and first child }
           1,2 :    => 9904;    { Between birth or union and other events }
             3 :  3 => 9905;    { Between children }
             3 :    => 9906;    { Between last child and later events }
               :    => 9907;    { Between other events }
        endbox;
        if eno = 9905 then
          errmsg(eno,norder(i),i,lcmc(i),ucmc(i),cmc2m(lcmc(i)),cmc2y(lcmc(i)),cmc2m(ucmc(i)),cmc2y(ucmc(i)),
                       norder(j),j,lcmc(j),ucmc(j),cmc2m(lcmc(j)),cmc2y(lcmc(j)),cmc2m(ucmc(j)),cmc2y(ucmc(j)),interv(j)+cdelay(j));
        else
          errmsg(eno,i,lcmc(i),ucmc(i),cmc2m(lcmc(i)),cmc2y(lcmc(i)),cmc2m(ucmc(i)),cmc2y(ucmc(i)),
                       j,lcmc(j),ucmc(j),cmc2m(lcmc(j)),cmc2y(lcmc(j)),cmc2m(ucmc(j)),cmc2y(ucmc(j)),interv(j)+cdelay(j));
        endif;
        errflag(j) = default;
    else
      lcmc(j) = t;
    endif;
  enddo;

{ -------------------------------------------------------------------------- }

  { backward checking of minimum interval }
  do i = nevents while i >= 2 by (-1)
    j = i - 1;
    t = adjubi(lcmc(j),ucmc(j),lcmc(i),ucmc(i),interv(i)+cdelay(i));
    if t < 0 then
        x1 = type(j);
        x2 = type(i);
        box x1 : x2 => eno;
             1 :  2 => 9901;    { Between birth and first union }
             1 :  3 => 9902;    { Between birth and first child }
             2 :  3 => 9903;    { Between union and first child }
           1,2 :    => 9904;    { Between birth or union and other events }
             3 :  3 => 9905;    { Between children }
             3 :    => 9906;    { Between last child and later events }
               :    => 9907;    { Between other events }
        endbox;
        if eno = 9905 then
          errmsg(eno,norder(i),i,lcmc(i),ucmc(i),cmc2m(lcmc(i)),cmc2y(lcmc(i)),cmc2m(ucmc(i)),cmc2y(ucmc(i)),
                       norder(j),j,lcmc(j),ucmc(j),cmc2m(lcmc(j)),cmc2y(lcmc(j)),cmc2m(ucmc(j)),cmc2y(ucmc(j)),interv(i)+cdelay(i));
        else
          errmsg(eno,i,lcmc(i),ucmc(i),cmc2m(lcmc(i)),cmc2y(lcmc(i)),cmc2m(ucmc(i)),cmc2y(ucmc(i)),
                       j,lcmc(j),ucmc(j),cmc2m(lcmc(j)),cmc2y(lcmc(j)),cmc2m(ucmc(j)),cmc2y(ucmc(j)),interv(i)+cdelay(i));
        endif;
        errflag(j) = default;
    else
      ucmc(j) = t;
    endif;
  enddo;


{ -------------------------------------------------------------------------- }

  { Equalizing twins }
  umin = 9999;
  lmax = 0;
  ntwin = 0;
  do i = 1 while i <= cebnew+chadj
    j = i + 1;
    if special(twincode(j)) | twincode(j) <= 1 then
      if umin < lmax then
        errmsg(9920,j-1,lmax,umin,cmc2m(lmax),cmc2y(lmax),cmc2m(umin),cmc2y(umin))
      else
        do k = 1 while k <= ntwin
          lcmc(j-k) = lmax;
          ucmc(j-k) = umin;
        enddo;
      endif;
      umin = 9999;
      lmax = 0;
      ntwin = 0;
    endif;
    if !special(twincode(j)) & twincode(j) >= 1 then
      ntwin = ntwin + 1;
      if lcmc(j) > lmax then lmax = lcmc(j) endif;
      if ucmc(j) < umin then umin = ucmc(j) endif;
    endif;
  enddo;

{ -------------------------------------------------------------------------- }

  { Preparation for imputation }
  if doimp then
    do i = 1 while i <= nevents-1
      if special(twincode(i)) | twincode(i) <= 1 then
        do j = nevents while j > i by (-1)
          if special(twincode(j)) | twincode(j) <= 1 then
            jinterv = 0;
            if j = i + 1 | (!special(twincode(j-1)) & i = j-twincode(j-1)) then
              jinterv = interv(j) + cdelay(j);
            endif;
            gap=ucmc(i)+jinterv-lcmc(j);
            if gap < 0 | lcmc(i) > lcmc(j) | ucmc(i) > ucmc(j) |
            (j = i+1 & errflag(i) = default & errflag(i+1) = default) then
              gap = 0
            endif;
            gap2 = int(gap/2);
            xgap = 0;
            if gap <> gap2*2 then xgap = 1 endif;
            ucmc(i) = ucmc(i) - gap2;
            lcmc(j) = lcmc(j) + gap2;
            if xgap = 1 then
              if ucmc(i) > lcmc(i) then
                ucmc(i) = ucmc(i) - 1
              else
                if ucmc(j) > lcmc(j) then
                  lcmc(j) = lcmc(j) + 1
                else
                  if !evermarr | i <> 2 | j <> 3 then
                    errmsg(9918,i,lcmc(i),ucmc(i),cmc2m(lcmc(i)),cmc2y(lcmc(i)),cmc2m(ucmc(i)),cmc2y(ucmc(i)),
                                  j,lcmc(j),ucmc(j),cmc2m(lcmc(j)),cmc2y(lcmc(j)),cmc2m(ucmc(j)),cmc2y(ucmc(j)),
                                  jinterv);
                    errflag(i) = default
                  endif
                endif
              endif
            endif;
          endif;
        enddo;
      else
        lcmc(i) = lcmc(i-1);
        ucmc(i) = ucmc(i-1);
      endif;
    enddo;

{ -------------------------------------------------------------------------- }

    { Imputation - Random }

    { Impute date of birth of respondent }
    if lcmc(1) > ucmc(1) then
      errmsg(9919,1,lcmc(1),ucmc(1),cmc2m(lcmc(1)),cmc2y(lcmc(1)),cmc2m(ucmc(1)),cmc2y(ucmc(1)));
      cmc(1) = default
    elseif lcmc(1) = ucmc(1) then
      cmc(1) = lcmc(1)
    else
      cmc(1) = random(lcmc(1),ucmc(1))
    endif;
    { Fix imputed date if a month given }
    if smonth(1) & (smonth(1) <> ((cmc(1) - 1) % 12) + 1) then
      x = int((cmc(1) - 1) / 12) * 12 + smonth(1);
      if x > ucmc(1) then x = x - 12 endif;
      if x < lcmc(1) then x = x + 12 endif;
      if x <= ucmc(1) then
        cmc(1) = x
      else
        errmsg(9921,1,smonth(1),lcmc(1),ucmc(1),cmc2m(lcmc(1)),cmc2y(lcmc(1)),cmc2m(ucmc(1)),cmc2y(ucmc(1)),cmc(1),cmc2m(cmc(1)),cmc2y(cmc(1)));
        cmc(1) = int((lcmc(1)+ucmc(1))/2);
      endif;
    endif;

{ -------------------------------------------------------------------------- }

    { Imputation - Random }

    do i = 2 while i <= nevents
      if lcmc(i) > ucmc(i) then
        { Inconsistent date }
        cmc(i) = default;
        errmsg(9919,i,lcmc(i),ucmc(i),cmc2m(lcmc(i)),cmc2y(lcmc(i)),cmc2m(ucmc(i)),cmc2y(ucmc(i)))
      elseif lcmc(i) = ucmc(i) then
        { Exact date }
        cmc(i) = lcmc(i)
      else
        { Imputed date }
        cmc(i) = random(lcmc(i),ucmc(i));
        if i > chadj & i <= cebnew+chadj & twincode(i) >= 1 then
          { Set same date for twins }
          do j = i+1 while j <= cebnew+chadj & twincode(j) > twincode(i)
            lcmc(j) = cmc(i);
            ucmc(j) = cmc(i);
          enddo;
        endif;
      endif;

      { Fix imputed date if a month given }
      if smonth(i) & (smonth(i) <> ((cmc(i) - 1) % 12) + 1) then
        x = int((cmc(i) - 1) / 12) * 12 + smonth(i);
        if x > ucmc(i) then x = x - 12 endif;
        if x < lcmc(i) then x = x + 12 endif;
        if x <= ucmc(i) then
          cmc(i) = x
        else
          errmsg(9921,i,smonth(i),lcmc(i),ucmc(i),cmc2m(lcmc(i)),cmc2y(lcmc(i)),cmc2m(ucmc(i)),cmc2y(ucmc(i)),cmc(i),cmc2m(cmc(i)),cmc2y(cmc(i)));
          cmc(i) = int((lcmc(i)+ucmc(i))/2);
        endif;
      endif;

{ -------------------------------------------------------------------------- }

      { Impute age at death if needed, otherwise update hot deck }
      if type(i) = 3 & BH5(norder(i)) = 2 then

        { Hot Deck entries: }
        { LSTDU: Only elements 1-10 are used, representing birth orders 1-9,10+ }
        { LSTDN: Elements  1-10: duration in days   for birth orders 1-9,10+ }
        {        Elements 11-20: duration in months for birth orders 1-9,10+ }
        {        Elements 21-30: duration in years  for birth orders 1-9,10+ }

        j = i;
        if i > 10 then j = 10 endif;

        if BH9A(norder(i)) = 9 | special(BH9A(norder(i))) then
          du = lstdu(j);
          dn = lstdn(10*(du-1)+j);
          flagdth(i) = 8;
        else
          du = BH9A(norder(i));
          if valid(BH9B(norder(i))) then
            dn = BH9B(norder(i));
          else
            dn = lstdn(10*(du-1)+j);
            flagdth(i) = 7
          endif;
          lstdu(j) = du;
          lstdn(10*(du-1)+j) = dn;
        endif;
        box du => dmonths;
             1 => int(dn/30);
             2 => dn;
             3 => dn*12;
               => dn;
        endbox;
        if dmonths < deathl(i) | dmonths > deathu(i) then
          if flagdth(i) = 7 | flagdth(i) = 8 |
             (du = 3 & dmonths < deathl(i) & dmonths+11 >= deathl(i)) then
            dmonths = deathl(i)
          endif
        endif;
        deathl(i) = dmonths;
        deathu(i) = dmonths;
      endif;

    enddo;

{ -------------------------------------------------------------------------- }

    impWM8C = cmc(1);                     { CMC date of birth }
    impWM8F = sflag(1);                   { Flag for date of birth }
    impWM9C = int((di-impWM8C)/12);         { Computed age of respondent }
    if evermarr then
      impMA6C = cmc(2);                   { CMC date of first union }
      impMA6F = sflag(2);                 { Flag for date of first union }
      impMA8C = int((cmc(2)-cmc(1))/12);  { Computed age at first union }
    else
      impMA6C = notappl;
      impMA6F = notappl;
      impMA8C = notappl;
    endif;
    do i = 1 while i <= cebnew
      j = i + chadj;
      impBH4C(i) = cmc(j);                { CMC date of birth of child }
      impBH4F(i) = sflag(j);              { Flag for date of birth of child }
      if BH5(i) = 2 then
        impBH9C(i) = deathl(j);           { Imputed age at death }
        impBH9F(i) = flagdth(j);          { Flag for age at death }
      else
        impBH9C(i) = notappl;
        impBH9F(i) = notappl;
      endif;
      export to BHimpute
        HH1,HH2,LN,BH1(i),impWM8C,impWM8F,impWM9C,impMA6C,impMA6F,impMA8C,impBH4C(i),impBH4F(i),impBH5(i),impBH9C(i),impBH9F(i),BH9A(i),BH9B(i);
    enddo;

  endif;      { end of imputation if no errors }

{ -------------------------------------------------------------------------- }

  { Check for errors and display event table }
  e = 0;
  do i = 1 while i <= nevents
    if errflag(i) = default then
      e = 1
    endif;
  enddo;
  if e then
    write( " " );
    write( "Cluster %02d    Household %03d      Woman's Line %02d", HH1, HH2, LN );
    write( " " );
    write( "                         Minimum       Maximum         Con Age   ");
    write( "Ix  T Ord  M  S  F  E   Date    CMC   CMC   Date   Int cep death f");

    do i = 1 while i <= nevents
      if type(i) = 3 & surv(i) = 2 then
        xdeath = BH9A(norder(i))*100;
        if BH9B(norder(i)) = missing then
          xdeath = xdeath+99
        else
          xdeath = xdeath+BH9B(norder(i))
        endif;
        sdeath = edit("ZZ999",xdeath);
      else
        xdeath = notappl;
        sdeath = "     ";
      endif;
      xdatemin = ((lcmc(i)-1)%12 + 1)*10000 + int((lcmc(i)-1)/12)+1900;
      xdatemax = ((ucmc(i)-1)%12 + 1)*10000 + int((ucmc(i)-1)/12)+1900;
      if errflag(i) = default then
        asterisk = "*"
      else
        asterisk = " ";
      endif;
      if type(i) = 3 then
        write( "%2d%3d%4d%3d%3d%3d%3s%8d%6d%6d%8d%5d%3d%5s%3d",
                i, type(i), norder(i),   twincode(i), surv(i),    sflag(i),
                asterisk,   xdatemin,    lcmc(i),     ucmc(i),    xdatemax,
                interv(i),  cdelay(i),   sdeath,     flagdth(i) );
      else
        write( "%2d%3d%10s%3d%3s%8d%6d%6d%8d%5d",
                i, type(i), spaces,      sflag(i),    asterisk,    xdatemin,    lcmc(i),
                ucmc(i),    xdatemax,    interv(i) );
      endif;

    enddo;

    write( "%s", linestr );
  endif;

endif;  { end of complete interview }

